import {Schema} from '../schema'
import ObjectId from '../object-id'
import Complex from '../complex'
import toMongooseSchema from '../mongoose-output'
import toTypescriptSchema from '../ts-output'
import toJSONSchema from '../json-schema-output'
import now from '../now'
import {R2Author, R2IntroductionComment} from './r2-common'
import {R2Comment} from './r2-comment'

const R2TestSet = new Complex({
  testSet: String,
  active: Boolean
})

const schema: Schema = {
  fields: {
    _id: ObjectId,
    introductionPositionId: String,
    status: {type: String, mongooseDefault: 'open', enum: ['open', 'closed', 'missed']},
    type: {
      type: String,
      mongooseDefault: 'unknown',
      enum: ['unknown', 'staffing', 'recruitment', 'recruitment-research', 'outsourcing', 'internal']
    },
    profession: String,
    n: {type: Number, mongooseDefault: 1},
    begin: Date,
    end: Date,
    createdAt: Date,
    description: String,
    location: String,
    createdBy: R2Author,
    owner: R2Author,
    owners: [R2Author],
    industry: String,
    company: {type: ObjectId, mongooseRef: 'Company', index: true},
    companyName: String,
    travelTimeMinutes: Number,
    statusChanges: [
      new Complex({
        timestamp: {type: Date, mongooseDefault: now},
        toStatus: String,
        author: R2Author
      })
    ],
    pinnedBy: Object, // in the form of { '001122330011223300112233': true, '001122330011223300112244': false } because we need to sort by the user's pinnings
    unhandledApplicationCount: {type: Number, mongooseDefault: 0},
    lastModified: Date,
    searchKeys: new Complex({
      en: [String],
      fi: [String]
    }),
    billingInfo: {type: ObjectId, mongooseRef: 'BillingInfo', optional: true, index: false},
    billingDetailsFrozen: Boolean,
    finderCity: {type: ObjectId, mongooseRef: 'FinderCity', optional: true, index: false},
    notes: String,
    rbnIntegration: new Complex({
      rbnId: {type: String, index: true, optional: true},
      interviewName: {type: String, optional: true},
      activated: Boolean
    }),
    reporting: [{type: ObjectId, mongooseRef: 'CommissionReport', optional: true, index: false}],
    integrationPointId: Object,
    comments: [R2Comment],
    aptitudeTestSets: new Complex({
      initial: R2TestSet,
      advanced: R2TestSet,
      evaluationProfile: {type: String, optional: true}
    }),
    introductionEvents: new Complex({
      timestamps: [
        new Complex({source: ObjectId, authorId: ObjectId, date: Date})
        ],
      lastSeen: [
        new Complex({
          user: ObjectId,
          date: Date
        })
      ]
    }),
    introductionComments: [R2IntroductionComment],
    summary: new Complex({
      openJobPostings: Number,
      numCandidates: Number,
      numUnevaluated: Number,
      numInterviewed: Number,
      numPlannedInterviews: Number,
      numIntroductions: Number,
      numSelected: Number
    }),
    lowercase: new Complex({
      profession: String,
      companyName: String,
      location: String
    }),
    recruitingPermissions: [
      new Complex({
        email: String,
        sent: Date,
        language: String,
        verificationCode: {type: String, index: 'unique-sparse'},
        decision: {type: Boolean, optional: true},
        answered: {type: Date, optional: true}
      })
    ],
    rbnOneIds: [String],
    customerId: {type: String, index: true}
  }
}

export default {
  get mongoose() {
    return toMongooseSchema('commissionMongooseSchema', schema)
  },
  get typescript() {
    return toTypescriptSchema('Commission', schema, 'typescript', { omitExtraExports: true })
  },
  get json() {
    return toJSONSchema('ExampleJSONSchema', schema, false)
  }
}
